<?php
namespace Myquest\Controller;

use Doctrine\DBAL\Connection;
use chillerlan\QRCode\QRCode;
// use chillerlan\QRCode\QROptions;
use Util\Utilities;
use Zeedhi\Framework\DataSource\DataSet;
use Myquest\Service\SMTP;

class PratoController
{

    protected $entityManager;
    protected $smtpService;
    private $qrCode;
    private $cripto;

    public function __construct($novoCodigo, $entityManager, $smtpService, $cripto)
    {
        $this->novoCodigo = $novoCodigo;
        $this->entityManager = $entityManager;
        $this->smtpService = $smtpService;
        $this->cripto = $cripto;
    }

    public function getPratos($request, $response)
    {
        $sql = "SELECT TO_NUMBER(NRORDPRATCAR) AS NRORDPRATCAR, CASE WHEN IDCONCNFMES = 'S' AND IDRESERVAMYMENU = 'S' THEN 1 WHEN IDRESERVAMYMENU = 'S' THEN 2 ELSE 3 END AS SESSAO_LAYOUT, CDTIPOPRAT, NMTIPOPRAT, NRSEQPRATCAR, CDFILAUXPRAT, CDPRATO, NMPRATO, DSPRATO, VRAVALIACAO, QTAVALIACAO, IDGLUTEN, IDLACTOSE, CALORIAS_KCAL_ROUND, NRFOTOPRATOBLOB, NRCARDAPIO, IDRESERVAMYMENU, CDMEDICASE, NMMEDICASE, QTMEDICASE,URL_FOTOPRATO FROM(SELECT P.CDTIPOPRAT, P.NMTIPOPRAT,P.NRSEQPRATCAR, NRORDPRATCAR, P.IDCONCNFMES, P.CDFILAUXPRAT, P.CDPRATO, P.NMPRATO, P.DSPRATO, P.VRAVALIACAO, P.QTAVALIACAO, P.IDGLUTEN, P.IDLACTOSE, TO_CHAR(P.CALORIAS_KCAL, '9999990.99') AS CALORIAS_KCAL_ROUND, NRFOTOPRATOBLOB, P.URL_FOTOPRATO,P.NRCARDAPIO, P.IDRESERVAMYMENU, P.CDMEDICASE, P.NMMEDICASE, P.QTMEDICASE FROM TABLE(PRATOS_MYMENU(NULL, NULL, :CDFILIAL, :CDSERVFILI, 'N', NULL, :DTCARDAPIO, :NRORG)) P) ORDER BY SESSAO_LAYOUT ASC, TO_NUMBER(NRORDPRATCAR)";
        $conn = $this
            ->entityManager
            ->getConnection();
        $row = $request->getRow();
        $params = [
            'NRORG' => $row['NRORG'], 
            'CDFILIAL' => $row['CDFILIAL'], 
            'CDSERVFILI' => $row['CDSERVFILI'], 
            'DTCARDAPIO' => $row['DTCARDAPIO']
        ];

        $statement = $conn->prepare($sql);
        $statement->execute($params);

        $result = $statement->fetchAll();
        $response->addDataSet(new DataSet('prato', $result));
    }

    public function insertReservaPrato($request, $response)
    {
        
        $sql = "INSERT INTO AVALIACAOPRATO (NRAVALIACAOPRATO, CDFILIAL, CDSERVFILI, NRCARDAPIO, NRSEQPRATCAR, CDFILAUXPRAT, CDPRATO, CDTIPOPRAT, DTCARDAPIO, NRORG, IDCONSUMO, NMUSUARIO, HRCONSUMO, DSCONSUMO, NRLOCALPESQ, CDREGISTROPESSOA, NRUSUARIOAPP, IDSTATUSRESERVA) VALUES (:NRAVALIACAOPRATO, :CDFILIAL, :CDSERVFILI, :NRCARDAPIO, :NRSEQPRATCAR, :CDFILAUXPRAT, :CDPRATO, :CDTIPOPRAT, :DTCARDAPIO, :NRORG, :IDCONSUMO, :NMUSUARIO, :HRCONSUMO, :DSCONSUMO, :NRLOCALPESQ, :CDREGISTROPESSOA, :NRUSUARIOAPP, 'AGENDADO')";
        $conn = $this
            ->entityManager
            ->getConnection();
        $row = $request->getRow();
        
        $CDFILIAL = $row["CDFILIAL"];
        $CDSERVFILI = $row["CDSERVFILI"];
        $DTCARDAPIO = $row["DTCARDAPIO"];
        $NRORG = $row["NRORG"];
        $NMUSUARIO = $row["NMUSUARIO"];
        $HRCONSUMO = $row["HRCONSUMO"];
        $DSCONSUMO = $row["DSCONSUMO"];
        $NRLOCALPESQ = $row["NRLOCALPESQ"];
        $plates = $row["plates"];
        $platesReserved = array();
        
        foreach($plates as $plate){
                    
            $NRAVALIACAOPRATO = $this
                            ->novoCodigo
                            ->getCodeByTableName('AVALIACAOPRATO', $row['NRORG'], 1, 10, false);
            $params = [
                "NRAVALIACAOPRATO" => $NRAVALIACAOPRATO ,
                "CDFILIAL" => $CDFILIAL,
                "CDSERVFILI" => $CDSERVFILI,
                "NRCARDAPIO" => $plate["NRCARDAPIO"],
                "NRSEQPRATCAR" => $plate["NRSEQPRATCAR"],
                "CDFILAUXPRAT" => $plate["CDFILAUXPRAT"],
                "CDPRATO" => $plate["CDPRATO"],
                "CDTIPOPRAT" => $plate["CDTIPOPRAT"],
                "CDREGISTROPESSOA" => $row["CDREGISTROPESSOA"],
                "NRUSUARIOAPP" => $row["NRUSUARIOAPP"],
                "DTCARDAPIO" => $DTCARDAPIO,
                "NRORG" => $NRORG,
                "IDCONSUMO" => "S",
                "NMUSUARIO" => $NMUSUARIO,
                "HRCONSUMO" => $HRCONSUMO,
                "DSCONSUMO" => $DSCONSUMO,
                "NRLOCALPESQ" => $NRLOCALPESQ
            ];
            $platesReserved[] = $NRAVALIACAOPRATO;
            $statement = $conn->prepare($sql);
            $statement->execute($params);
        }
        
        $result = [
            'NRORG'              => $row['NRORG'],
            'type'               => 'P',
            'NRAVALIACAOPRATO'   => $platesReserved
        ];
        $response->addDataSet(new DataSet('data', $result));
    }
    
    public function sendReservPlatesMail($request, $response) {
        $row = $request->getRow();
        $body = $this->constructPlatesEmail($row);
        $address = $row['DSEMAILUSUARIOAPP'];
        $this->smtpService->setSubject('Dados da reserva.');
        // caso seja necessário no futuro enviar algo por anexo, apenas seguir o exemplo abaixo
        // $this->smtpService->addAttachment('qrcode.png', 'QrCode Reserva');
        // Caso não encontre no banco, usa os mnemonicos
        $smtpParametros = $this->getFilial($row);
        if(!empty($smtpParametros)){
            $paramsAux = Utilities::getSmtpSettingsFromAddress($smtpParametros['host']);
            $smtpParametros['host'] = $paramsAux['host'];
            $smtpParametros['port'] = $paramsAux['port'] ?? null;
            $smtpParametros['password'] = ($smtpParametros['password']) ? $this->cripto->unCripto($smtpParametros['password'], 2) : '';
            $SMTPServiceAux = new SMTP($smtpParametros);
            $this->smtpService = $SMTPServiceAux;
        }
        $this->smtpService->sendMail($body, $address);   
    }
    
    private function getFilial($row){

        $query = "
               SELECT 
                NVL(DSLOGINSMTP, DSEMAILFILI) AS username,
                DSSMTPEMAILF AS host,
                CDSENHEMAILF AS password,
                CASE
                    WHEN IDUTTLSEMAIL = 'S' 
                    THEN 'tls' 
                    WHEN IDUTSSLEMAIL = 'S' 
                    THEN 'ssl' 
                    ELSE '' 
                END AS smtpSecure, 
                NVL(DSEMAILFILI, DSLOGINSMTP) AS email_From,
                CASE 
                    WHEN NVL(DSLOGINSMTP, DSEMAILFILI) IS NOT NULL AND 
                    CDSENHEMAILF IS NOT NULL 
                    THEN 'S' 
                    ELSE 'N' 
                    END AS auth
                FROM ENDEFILI 
                WHERE CDFILIAL = :CDFILIAL 
                AND IDTPENDEFILI = 'P' AND
                DSSMTPEMAILF IS NOT NULL
        ";
        
        
        $params = [
            "CDFILIAL" => $row['CDFILIAL']
        ];
        
        try{
            $conn = $this->entityManager->getConnection();
            $std = $conn->prepare($query);
            $std->execute($params);
            $rowResult = $std->fetch();
            
            if(!empty($rowResult)){
                $rowResult['username'] = $rowResult['USERNAME'];
                $rowResult['host'] = $rowResult['HOST'];
                $rowResult['password'] = $rowResult['PASSWORD'];
                $rowResult['smtpSecure'] = $rowResult['SMTPSECURE'];
                $rowResult['from'] = $rowResult['EMAIL_FROM'];
                $rowResult['smtpAuth'] = $rowResult['AUTH'];
                    
                unset($rowResult['USERNAME']);
                unset($rowResult['HOST']);
                unset($rowResult['PASSWORD']);
                unset($rowResult['SMTPSECURE']);
                unset($rowResult['EMAIL_FROM']);
                unset($rowResult['AUTH']);
                
                if($rowResult['smtpAuth'] == 'S')
                    $rowResult['smtpAuth'] = true;
                else
                    $rowResult['smtpAuth'] = false;
            }
            return $rowResult;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function constructPlatesEmail($row) {
        $name = $row['NMUSUARIO'];
        $plates = $row['plates'];
        $allPlates = '';
        foreach ($plates as $plate) {
            $allPlates .= $plate['NMPRATO'] . ', ';
        }
        $allPlates = rtrim($allPlates, ', ');
        $serviceDate = $row['DTCARDAPIO'];
        $serviceTime = substr($row['HRCONSUMO'], 0, 2) . ":" . substr($row['HRCONSUMO'], 2);
        if ($row['digitalReservation'] === 'true') {
            $data = $row['NRORG'] . ',' . $row['type'] . ',' . implode(',', $row['NRAVALIACAOPRATO']);
            $qrCodeBase64 = (new QRCode)->render($data);
            $qrCode = '<img src="'.$qrCodeBase64.'" alt="QR Code" />';
        } else {
            $qrCode = '';
        }
        switch ($row['lang']) {
            case 'pt-br':
                $mailTemplate = file_get_contents(__DIR__ . "/../Templates/reservation-platesPT-mail.html");
                if ($row['digitalReservation'] === 'true') {
                    $footerMessage = 'Para realizar a retirada do seu pedido informe seu CPF cadastrado na reserva ou faça a leitura do Qr-Code abaixo no aplicativo MyQuest, disponibilizado no restaurante.';
                } else {
                    $footerMessage = '';
                }
                break;
            case 'en-us':
                $mailTemplate = file_get_contents(__DIR__ . "/../Templates/reservation-platesEN-mail.html");
                if ($row['digitalReservation'] === 'true') {
                    $footerMessage = 'To collect your order, enter your ID number registered when booking or scan the Qr-Code below in the MyQuest application, available in the restaurant.';
                } else {
                    $footerMessage = '';
                }
                break;
            case 'es-es':
                $mailTemplate = file_get_contents(__DIR__ . "/../Templates/reservation-platesES-mail.html");
                if ($row['digitalReservation'] === 'true') {
                    $footerMessage = 'Para recoger tu pedido introduce tu número de ID registrado al hacer la reserva o escanea el código Qr que aparece a continuación en la aplicación MyQuest, disponible en el restaurante.';
                } else {
                    $footerMessage = '';
                }
                break;
            default:
                $mailTemplate = file_get_contents(__DIR__ . "/../Templates/reservation-servicePT-mail.html");
                if ($row['digitalReservation'] === 'true') {
                    $footerMessage = 'Para realizar a retirada do seu pedido informe seu CPF cadastrado na reserva ou faça a leitura do Qr-Code abaixo no aplicativo MyQuest, disponibilizado no restaurante.';
                } else {
                    $footerMessage = '';
                }
                break;
        }
        $logo = $row['logo'];
        $content = mb_convert_encoding($mailTemplate, 'UTF-8', mb_detect_encoding($mailTemplate, 'UTF-8, ISO-8859-1', true));
        $search = array(
            "%logo%",
            "%name%",
            "%service%",
            "%serviceDate%",
            "%serviceTime%",
            "%footerMessage%",
            "%qrCode%",
        );
        $replace = array(
            $logo,
            $name,
            $allPlates,
            $serviceDate,
            $serviceTime,
            $footerMessage,
            $qrCode,
        );
        
        return str_replace($search, $replace, $content);
    }
    
    public function ratePlate($request, $response)
    {
        $sql = "INSERT INTO AVALIACAOPRATO (NRAVALIACAOPRATO, 
                                            IDCONSUMO, 
                                            CDFILIAL, 
                                            CDSERVFILI, 
                                            NRCARDAPIO, 
                                            DTCARDAPIO, 
                                            NRORG, 
                                            NRLOCALPESQ, 
                                            CDPRATO, 
                                            CDFILAUXPRAT, 
                                            VRAVALIACAOPRATO,
                                            CDTIPOPRAT,
                                            NRSEQPRATCAR) 
                            VALUES (:NRAVALIACAOPRATO, 
                                    :IDCONSUMO, 
                                    :CDFILIAL, 
                                    :CDSERVFILI, 
                                    :NRCARDAPIO, 
                                    :DTCARDAPIO,
                                    :NRORG,
                                    :NRLOCALPESQ, 
                                    :CDPRATO, 
                                    :CDFILAUXPRAT, 
                                    :VRAVALIACAOPRATO,
                                    :CDTIPOPRAT,
                                    :NRSEQPRATCAR)";
                                    
        $conn = $this->entityManager->getConnection();
        $row = $request->getRow();
        
        $NRAVALIACAOPRATO = $this->novoCodigo->getCodeByTableName('AVALIACAOPRATO', $row['NRORG'], 1, 10, false);
                        
        $params = [
            'NRAVALIACAOPRATO' => $NRAVALIACAOPRATO,
            'IDCONSUMO' => "N",
            'CDFILIAL' => $row['CDFILIAL'],
            'CDSERVFILI' => $row['CDSERVFILI'],
            'NRCARDAPIO' => $row['NRCARDAPIO'],
            'DTCARDAPIO' => $row['DTCARDAPIO'],
            'NRORG' => $row['NRORG'],
            'NRLOCALPESQ' => $row['NRLOCALPESQ'],
            'CDPRATO' => $row['CDPRATO'],
            'CDFILAUXPRAT' => $row['CDFILAUXPRAT'],
            'VRAVALIACAOPRATO' => $row['rateValue'],
            'CDTIPOPRAT' => $row['CDTIPOPRAT'],
            'NRSEQPRATCAR' => $row['NRSEQPRATCAR']
        ];

        $statement = $conn->prepare($sql);
        $statement->execute($params);
    }
    
    public function getNutrientes($request, $response){

        $sql = "SELECT G.CDGRUPNUTR, G.NMGRUPNUTR, N.CDNUTRIENTE, N.NMNUTRIENTE, N.NMREDUZIDO, TO_CHAR(NUT.QTNUTRIENTE, '9999990.9999') AS QTNUTRIENTE , N.SGUNIDADE FROM ( SELECT CDNUTRCALC, SUM(P.QTNUTRCALC) AS QTNUTRIENTE FROM TABLE ( NUTRIENTE_PRATO_PME(:CDFILCENTPME,:CDTIPOCARD,:CDFILIAL,:CDSERVFILI,NULL,'S',:CDPRATO,:CDPRATO) ) P WHERE :IDTIPOSERV = 'M' GROUP BY CDNUTRCALC UNION ALL SELECT CDNUTRCALC, SUM(P.QTNUTRCALC) AS QTNUTRIENTE FROM TABLE ( NUTRIENTE_PRATO(:CDFILIAL,:CDSERVFILI,NULL,'S',:CDPRATO,:CDPRATO) ) P WHERE :IDTIPOSERV <> 'M' GROUP BY CDNUTRCALC ) NUT INNER JOIN NUTRIENTE N ON NUT.CDNUTRCALC = N.CDNUTRIENTE AND NRORG = :NRORG INNER JOIN GRUPNUTR G ON G.CDGRUPNUTR = N.CDGRUPNUTR ORDER BY G.CDGRUPNUTR, N.CDNUTRIENTE";

        $conn = $this
            ->entityManager
            ->getConnection();
        $row = $request->getRow();
        
        $params = array(
            "CDPRATO" => $row['CDPRATO'],
            "NRORG" => $row['NRORG'],
            "CDSERVFILI" => $row['CDSERVFILI'],
            "IDTIPOSERV" => $row['IDTIPOSERV'],
            "CDFILIAL" => $row['CDFILIAL'],
            "CDFILCENTPME" => null,
            "CDTIPOCARD" => null
        );
        
        $statement = $conn->prepare($sql);
        $statement->execute($params);

        $result = $statement->fetchAll();
        $response->addDataSet(new DataSet('nutrientes', $result));
    }

    protected function executeInsertSql($sql, $params = null)
    {
        $conn = $this->entityManager->getConnection();
        $statement = $conn->prepare($sql);
        $statement->execute($params);
    }
}